import { GraphicsDevice } from '../core/graphics-device.js'

interface Listener {
  (...args: any): void
}

export class EventEmitter {
  _listeners = {}

  addEventListener(type: PropertyKey, listener: Listener, options?: object): void {
    this.on(type, listener, options)
  }

  removeEventListener(type: PropertyKey, listener: Listener, options?: object): void {
    this.off(type, listener, options)
  }

  dispatchEvent(event: Event): boolean {
    const details = 'detail' in event ? event.detail : undefined
    return this.emit(event.type, ...Object.values(details))
  }

  on(event: PropertyKey, callback: Listener, _options?: object) {
    this._listeners[event] = this._listeners[event] || []
    this._listeners[event].push(callback)
  }

  emit(event: PropertyKey, ...args: any[]) {
    const listeners = this._listeners[event]

    if (listeners) {
      listeners.forEach(
        (cb: Listener) => cb(...args)
      )
    }

    return true
  }

  off(event: PropertyKey, callback: Listener, _options?: object) {
    const listeners = this._listeners[event]

    if (listeners) {
      this._listeners[event] = listeners.filter(
        (cb: Listener) => cb !== callback
      )
    }
  }
}


export class GraphicsDeviceInitialized {
  static EventName = 'graphics-device:initialized'
  graphicsDevice: GraphicsDevice

  constructor(graphicsDevice: GraphicsDevice) {
    this.graphicsDevice = graphicsDevice
  }
}

export class GraphicsDeviceLost {
  static EventName = 'graphics-device:device-lost'
  info: GPUDeviceLostInfo

  constructor(info: GPUDeviceLostInfo) {
    this.info = info
  }
}


